#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
require("source-map-support/register");
const chalk_1 = require("chalk");
const exec = require("execa");
const fs_1 = require("mz/fs");
const prompt = require("./prompt");
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(['', 'Welcome to the Sourcegraph extension creator!', ''].join('\n'));
        if (!(yield fs_1.exists('.git'))) {
            console.log('📘 .git directory not found, initilizing git repository');
            yield exec('git', ['init']);
        }
        let name;
        let title;
        let description;
        let publisher;
        try {
            ;
            ({ name, title, description, publisher } = JSON.parse(yield fs_1.readFile('package.json', 'utf-8')));
        }
        catch (err) {
            if (err.code !== 'ENOENT') {
                throw err;
            }
        }
        if (name) {
            console.log(`Extension name is "${name}"`);
        }
        else {
            name = yield prompt.input({
                message: 'What should the name of the extension be (kebab-case)?',
            });
        }
        if (title) {
            console.log(`Extension title is "${title}"`);
        }
        else {
            title = yield prompt.input({
                message: 'What should the title of the extension be (Sentence case)?',
            });
        }
        if (description) {
            console.log(`Description is "${description}"`);
        }
        else {
            description = yield prompt.input({
                message: 'What does this extension do?',
            });
        }
        if (publisher) {
            console.log(`Publisher is "${publisher}"`);
        }
        else {
            publisher = yield prompt.input({
                message: 'What is your Sourcegraph organization or username?',
                default: 'SOURCEGRAPH_ORG_OR_USERNAME',
            });
        }
        const licenseName = yield prompt.choices({
            message: 'License?',
            choices: ['UNLICENSED', 'MIT'],
            default: 'MIT',
        });
        if (yield fs_1.exists('tsconfig.json')) {
            console.log('📄 tsconfig.json already exists, skipping creation');
        }
        else {
            const tsconfigJson = {
                extends: './node_modules/@sourcegraph/tsconfig/tsconfig.json',
                compilerOptions: {
                    target: 'es2016',
                    module: 'esnext',
                    moduleResolution: 'node',
                    sourceMap: true,
                    declaration: true,
                    outDir: 'dist',
                    rootDir: 'src',
                    esModuleInterop: true,
                    allowSyntheticDefaultImports: true,
                },
            };
            console.log('📄 Adding tsconfig.json');
            yield fs_1.writeFile('tsconfig.json', JSON.stringify(tsconfigJson, null, 2));
        }
        if (yield fs_1.exists('tslint.json')) {
            console.log('📄 tslint.json already exists, skipping creation');
        }
        else {
            console.log('📄 Adding tslint.json');
            const tslintJson = {
                extends: ['@sourcegraph/tslint-config'],
            };
            yield fs_1.writeFile('tslint.json', JSON.stringify(tslintJson, null, 2));
        }
        console.log('📄 Adding .editorconfig');
        yield fs_1.writeFile('.editorconfig', [
            '[*]',
            'insert_final_newline = true',
            'end_of_line = lf',
            'charset = utf-8',
            'trim_trailing_whitespace = true',
            'indent_style = space',
            'indent_size = 4',
            '',
            '[*.{json,js,yml}]',
            'indent_size = 2',
            '',
            '[*.md]',
            'trim_trailing_whitespace = false',
            '',
        ].join('\n'));
        console.log('📄 Adding .gitignore');
        yield fs_1.writeFile('.gitignore', ['dist/', 'node_modules/', ''].join('\n'));
        if (yield fs_1.exists('package.json')) {
            console.log('📄 package.json already exists, skipping creation');
        }
        else {
            console.log('📄 Adding package.json');
            const packageJson = {
                name,
                title,
                description,
                publisher,
                activationEvents: ['*'],
                contributes: {
                    actions: [],
                    menus: {
                        'editor/title': [],
                        commandPalette: [],
                    },
                    configuration: {},
                },
                version: '0.0.0-DEVELOPMENT',
                license: licenseName,
                main: `dist/${name}.js`,
                scripts: {
                    tslint: "tslint -p tsconfig.json './src/**/*.ts'",
                    typecheck: 'tsc -p tsconfig.json',
                    build: `parcel build --out-file dist/${name}.js src/${name}.ts`,
                    serve: `parcel serve --no-hmr --out-file dist/${name}.js src/${name}.ts`,
                    'watch:typecheck': 'tsc -p tsconfig.json -w',
                    'watch:build': 'tsc -p tsconfig.dist.json -w',
                    'sourcegraph:prepublish': 'npm run build',
                },
                browserslist: [
                    'last 1 Chrome versions',
                    'last 1 Firefox versions',
                    'last 1 Edge versions',
                    'last 1 Safari versions',
                ],
            };
            yield fs_1.writeFile('package.json', JSON.stringify(packageJson, null, 2));
        }
        try {
            console.log('📂 Creating src directory');
            yield fs_1.mkdir('src');
            yield fs_1.writeFile(`src/${name}.ts`, [
                "import * as sourcegraph from 'sourcegraph'",
                '',
                'export function activate(): void {',
                "   sourcegraph.languages.registerHoverProvider(['*'], {",
                `       provideHover: () => ({ contents: { value: 'Hello world from ${title}! 🎉🎉🎉' } })`,
                '   })',
                '}',
                '',
                '// Learn what else is possible by visiting the [Sourcegraph extension documentation](https://github.com/sourcegraph/sourcegraph-extension-docs)',
                '',
            ].join('\n'));
        }
        catch (err) {
            if (err.code !== 'EEXIST') {
                throw err;
            }
        }
        console.log('📦 Installing dependencies');
        yield exec('npm', [
            'install',
            '--save-dev',
            'sourcegraph',
            'typescript',
            'parcel-bundler',
            'tslint',
            '@sourcegraph/tslint-config',
            '@sourcegraph/tsconfig',
        ], { stdio: 'inherit' });
        if (yield fs_1.exists('README.md')) {
            console.log('📄 README.md already exists, skipping creation.');
        }
        else {
            console.log('📄 Adding README');
            const readme = [
                `# ${title}`,
                '',
                description[description.length - 1] === '.' ? description : description + '.',
                '',
                '## Prerequisites',
                '',
                'Sourcegraph extensions are written in TypeScript and are distributed as bundled JavaScript files that run on the client. For creation, publishing, and viewing, you need:',
                '',
                '- **Creation**: Install [Node.js](https://nodejs.org).',
                '- **Publishing**: Install the [Sourcegraph CLI (`src`)](https://github.com/sourcegraph/src-cli#installation) and create a [Sourcegraph.com account](https://sourcegraph.com/sign-up).',
                '- **Viewing**: Install the Sourcegraph extension for [Chrome](https://chrome.google.com/webstore/detail/sourcegraph/dgjhfomjieaadpoljlnidmbgkdffpack) or [Firefox](https://addons.mozilla.org/en-US/firefox/addon/sourcegraph/).',
                '',
                '## Set up',
                '',
                '```',
                `npm install`,
                '```',
                '',
                '## Lint and type check',
                '',
                '```',
                `npm run tslint`,
                `npm run typecheck`,
                '```',
                '',
                '## Publish',
                '',
                '```',
                'src extensions publish',
                '```',
                '',
                '## Sourecgraph extension API',
                '',
                'Visit the [Sourcegraph extension documentation](https://github.com/sourcegraph/sourcegraph-extension-docs) and check out some [Sourcegraph extension samples](https://github.com/sourcegraph/sourcegraph-extension-samples).',
                '',
            ].join('\n');
            yield fs_1.writeFile('README.md', readme);
        }
        setTimeout(() => process.exit(0), 100);
    });
}
main().catch(err => {
    if (err.showStack === false) {
        console.error('\n' + chalk_1.default.red(err.message) + '\n');
    }
    else {
        console.error(err);
    }
    setTimeout(() => process.exit(1), 100);
});
//# sourceMappingURL=cli.js.map